@@ -1,7 +1,6 @@
 from webob import Request, Response
-from webob.exc import HTTPFound
+from webob.exc import HTTPException
 from authenticator import CookieAuthenticator
-from paste.registry import RegistryManager
 from mail_traceback import send_traceback, format_traceback
 import traceback, mimetypes, os
 from controller import request as a_request, response as a_response
@@ -25,10 +24,10 @@ class ArtichokeCore(object):
 
     def __call__(self, environ, start_response):
         request = Request(environ=environ)
-        environ['paste.registry'].register(a_request, request)
+        environ['artichoke.locals'].register(environ, a_request, request)
 
         response = Response(body="Artichoke Default Page", content_type='text/html', charset='utf-8')
-        environ['paste.registry'].register(a_response, response)
+        environ['artichoke.locals'].register(environ, a_response, response)
 
         if self.force_request_encoding:
             request.charset = self.force_request_encoding
@@ -43,7 +42,7 @@ class ArtichokeCore(object):
                 self.authenticator.authenticate(request)
                 self.root._dispatch(request)
                 self.authenticator.inject_cookie(response)
-            except HTTPFound, e:
+            except HTTPException, e:
                 response = e
                 self.authenticator.inject_cookie(response)
         return response(environ, start_response)
@@ -80,6 +79,22 @@ class ErrorMiddleware(object):
 
             return resp(environ, start_response)
             
+class ThreadLocalsManager(object):
+    def __init__(self, app):
+        self.app = app
+
+    def __call__(self, environ, start_response):
+        try:
+            environ['artichoke.locals'] = self
+            environ['artichoke.registered_locals'] = []
+            return self.app(environ, start_response)
+        finally:
+            for ri in environ['artichoke.registered_locals']:
+                ri._pop_object()
+
+    def register(self, environ, proxy, instance):
+        environ['artichoke.registered_locals'].append(proxy)
+        proxy._set_object(instance)
 
 class Application(object):
     def __init__(self, root, templates_path, config={}):
@@ -90,7 +105,7 @@ class Application(object):
             self.app = Middleware(self.app, self.core, config)
 
         self.app = ErrorMiddleware(self.app, self.core, config)
-        self.app = RegistryManager(self.app)
+        self.app = ThreadLocalsManager(self.app)
 
     def __call__(self, environ, start_response):
         return self.app(environ, start_response)
